@@ -3,7 +3,6 @@ from __future__ import annotations
 
 import abc
 import inspect
-import json
 import typing as t
 
 import marshmallow as ma
@@ -12,10 +11,11 @@ from muffin import Request
 from muffin.typing import JSONType
 from muffin.handler import Handler, HandlerMeta
 
-from . import FILTERS_PARAM, LIMIT_PARAM, OFFSET_PARAM, SORT_PARAM, openapi
+from . import LIMIT_PARAM, OFFSET_PARAM, openapi
 from .api import API
 from .errors import APIError
 from .filters import Filters, Filter
+from .sorting import Sorting, Sort
 
 
 T = t.TypeVar('T')
@@ -33,6 +33,9 @@ class RESTOptions:
     # Base class for filters
     filters_cls: t.Type[Filters] = Filters
 
+    # Base class for sorting
+    sorting_cls: t.Type[Sorting] = Sorting
+
     # Auto generation for schemas
     schema_base: t.Type[ma.Schema] = ma.Schema
     schema_fields: t.Dict = {}
@@ -41,7 +44,7 @@ class RESTOptions:
 
     if t.TYPE_CHECKING:
         filters: Filters
-        sorting: t.Dict[str, bool] = {}
+        sorting: Sorting
         Schema: t.Type[ma.Schema]
 
     def __init__(self, cls):
@@ -61,11 +64,6 @@ class RESTOptions:
         self.name = self.name or cls_name.lower().split('handler', 1)[0] or cls_name.lower()
         self.name_id = self.name_id or self.name
 
-        # Setup sorting
-        if not isinstance(self.sorting, dict):
-            self.sorting = dict(
-                n if isinstance(n, (list, tuple)) else (n, bool(n)) for n in self.sorting)
-
         # Setup schema
         if not self.Schema:
             self.Schema = type(
@@ -88,9 +86,12 @@ class RESTHandlerMeta(HandlerMeta):
         """Prepare options for the handler."""
         params.setdefault('Meta', type("Meta", (object,), {}))  # Every handler has uniq meta
         cls = super().__new__(mcs, name, bases, params)
+
         if not getattr(cls.Meta, 'abc', False):
             cls.meta = cls.meta_class(cls)
-            cls.meta.filters = cls.meta.filters_cls(*cls.meta.filters, handler=cls)
+            cls.meta.filters = cls.meta.filters_cls(cls, cls.meta.filters)
+            cls.meta.sorting = cls.meta.sorting_cls(cls, cls.meta.sorting)
+
         return cls
 
 
@@ -116,7 +117,7 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
         filters: t.Sequence[t.Union[str, t.Tuple[str, str], Filter]] = ()
 
         # Define allowed resource sorting params
-        sorting: t.Union[t.Dict[str, bool], t.Sequence[t.Union[str, t.Tuple[str, bool]]]] = {}
+        sorting: t.Sequence[t.Union[str, t.Tuple[str, t.Dict], Sort]] = ()
 
         # Serialize/Deserialize Schema class
         Schema: t.Optional[t.Type[ma.Schema]] = None
@@ -129,10 +130,9 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
             router.bind(cls, *paths, methods=methods, **params)
 
         else:
-            router.bind(cls, f"/{ cls.meta.name }",
-                        methods=methods & {'GET', 'POST', 'DELETE'}, **params)
+            router.bind(cls, f"/{ cls.meta.name }", methods=methods, **params)
             router.bind(cls, f"/{ cls.meta.name }/{{{ cls.meta.name_id }}}",
-                        methods=methods & {'GET', 'PUT', 'DELETE'}, **params)
+                        methods=methods, **params)
 
         for _, method in inspect.getmembers(cls, lambda m: hasattr(m, '__route__')):
             cpaths, cparams = method.__route__
@@ -149,32 +149,24 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
         if resource or request.method != 'GET':
             return await method(request, resource=resource)
 
-        # Filter the collection
-        filters = request.url.query.get(FILTERS_PARAM)
-        if filters:
-            try:
-                data = json.loads(filters)
-                _, self.collection = self.meta.filters.filter(data, self.collection, handler=self)
+        meta = self.meta
+        query = request.url.query
 
-            except (ValueError, TypeError):
-                self.api.logger.warning(f'Invalid filters data: { request.url }')
+        # Filter collection
+        if meta.filters:
+            self.collection = meta.filters.apply(request, self.collection, **options)
 
-        # Sort resources
-        if SORT_PARAM in request.url.query:
-            sorting = [
-                (name.strip('-'), name.startswith('-'))
-                for name in request.url.query[SORT_PARAM].split(',')
-                if name.strip('-') in self.meta.sorting]
-
-            self.collection = await self.sort(request, *sorting, **options)
+        # Sort collection
+        if meta.sorting:
+            self.collection = meta.sorting.apply(request, self.collection, **options)
 
         # Paginate the collection
         headers = {}
-        limit = request.url.query.get(LIMIT_PARAM) or self.meta.limit
-        if self.meta.limit and limit:
+        if meta.limit:
+            limit = query.get(LIMIT_PARAM) or meta.limit
             try:
-                limit = min(abs(int(limit)), self.meta.limit)
-                offset = int(request.url.query.get(OFFSET_PARAM, 0))
+                limit = min(abs(int(limit)), meta.limit)
+                offset = int(query.get(OFFSET_PARAM, 0))
                 if limit and offset >= 0:
                     self.collection, total = await self.paginate(
                         request, limit=limit, offset=offset)
@@ -190,6 +182,13 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
 
         return response
 
+    @property
+    def api(self) -> API:
+        """Check if the handler is binded to an API."""
+        if self._api is None:
+            raise Exception('The handler is not routed by any API')  # TODO
+        return self._api
+
     @abc.abstractmethod
     async def prepare_collection(self, request: Request) -> t.Any:
         """Prepare a collection of resources. Create queryset, db cursor and etc."""
@@ -201,10 +200,6 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
         """Paginate the results."""
         raise NotImplementedError
 
-    def paginate_prepare_headers(self, limit, offset, total):
-        """Prepare pagination headers."""
-        return {'x-total': total, 'x-limit': limit, 'x-offset': offset}
-
     @abc.abstractmethod
     async def save(self, request: Request, *, resource: T = None) -> T:
         """Save the given resource."""
@@ -215,17 +210,13 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
         """Remove the given resource."""
         raise NotImplementedError
 
-    @abc.abstractmethod
-    async def sort(self, request: Request, *sorting: t.Tuple[str, bool], **options) -> t.Any:
-        """Remove the given resource."""
-        raise NotImplementedError
+    async def prepare_resource(self, request: Request) -> t.Any:
+        """Load a resource."""
+        return request['path_params'].get(self.meta.name_id)
 
-    @property
-    def api(self) -> API:
-        """Check if the handler is binded to an API."""
-        if self._api is None:
-            raise Exception('The handler is not routed by any API')  # TODO
-        return self._api
+    def paginate_prepare_headers(self, limit, offset, total):
+        """Prepare pagination headers."""
+        return {'x-total': total, 'x-limit': limit, 'x-offset': offset}
 
     async def authorize(self, request: Request):
         """Default authorization method. Proxy auth to self.api."""
@@ -242,6 +233,27 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
             exclude=request.url.query.get('schema_exclude', ()),
         )
 
+    async def parse(self, request: Request):
+        """Parse data from the given request."""
+        try:
+            return await request.data(raise_errors=True)
+        except (ValueError, TypeError) as exc:
+            raise APIError.BAD_REQUEST(str(exc))
+
+    async def load(self, request: Request, *, resource: t.Any = None) -> t.Any:
+        """Load data from request and create/update a resource."""
+        data = await self.parse(request)
+        schema = await self.get_schema(request, resource=resource)
+        if not schema:
+            return data
+
+        try:
+            resource = schema.load(data, partial=resource is not None, many=isinstance(data, list))  # type: ignore # noqa
+        except ma.ValidationError as exc:
+            raise APIError.BAD_REQUEST('Invalid data', errors=exc.messages)
+
+        return resource
+
     async def dump(self, request: Request, response: t.Any, *, many=...) -> JSONType:
         """Serialize the given response."""
         schema = await self.get_schema(request)
@@ -283,28 +295,6 @@ class RESTBase(Handler, metaclass=RESTHandlerMeta):
 
         return await self.remove(request, resource=resource)
 
-    async def prepare_resource(self, request: Request) -> t.Any:
-        """Load a resource."""
-        return request['path_params'].get(self.meta.name_id)
-
-    async def load(self, request: Request, *, resource: t.Any = None) -> t.Any:
-        """Load data from request and create/update a resource."""
-        try:
-            data = await request.data()
-        except (ValueError, TypeError) as exc:
-            raise APIError.BAD_REQUEST(str(exc))
-
-        schema = await self.get_schema(request, resource=resource)
-        if not schema:
-            return data
-
-        try:
-            resource = schema.load(data, partial=resource is not None, many=isinstance(data, list))  # type: ignore # noqa
-        except ma.ValidationError as exc:
-            raise APIError.BAD_REQUEST('Invalid data', errors=exc.messages)
-
-        return resource
-
 
 class RESTHandler(RESTBase, openapi.OpenAPIMixin):
     """Basic Handler Class."""
@@ -313,3 +303,11 @@ class RESTHandler(RESTBase, openapi.OpenAPIMixin):
         """Tune the handler."""
 
         abc: bool = True                        # The class is abstract, meta wouldn't be generated
+
+
+def to_sort(sort_params: t.Sequence[str]) -> t.Generator[t.Tuple[str, bool], None, None]:
+    """Generate sort params."""
+    for name in sort_params:
+        n, desc = name.strip('-'), name.startswith('-')
+        if n:
+            yield n, desc
